// Global solutions to MRF in F0 with a graph cut

/***************** Library parameters ******************/
// Library parameters : comment if not suitable
// Use the graphical library, that needs to be installed first in that case ?
// Warning : every time you (un)comment this option, you have to update CMakeLists.txt accordingly and run cmake.
//#define I_want_to_use_the_graphical_library
// If your OS is Linux, you can have colors in terminals !
#define My_OS_is_GNU_Linux_and_I_like_colors


/***************** Including libraries *****************/
// Graphical library 
#ifdef I_want_to_use_the_graphical_library
 #include <Imagine/Graphics.h>
 #include <Imagine/Images.h>
 using namespace Imagine;
#endif

// If OS is linux, include first
// otherwise inlude second
#ifdef My_OS_is_GNU_Linux_and_I_like_colors
 #include "colors_in_terminal.h"
#else
 #include "no_colors_in_terminal.h";
#endif


// Personal library
#include "familial_permutations.h"
#include "images.h"

// With graphical library ?
#ifdef I_want_to_use_the_graphical_library
 #include "with_graphical_library.h"
 // should have a look at this one even if not using this graphical library :
 #include "requires_graphical_library.h"  
#endif


// Minimization tools
#include "minimization_F0.h"



/*************************************  The main function ***************************/

// Example of how to minimize a random energy or one based on an image
// First : minimization of a random energy in the family
// Second (if graphical library installed) : examples of image-based energies (select one of the 2 examples in the corresponding file)
// Check the functions called and update them to your own energies
int main(int argc, char **argv){


  /******** Minimizing a random energy *********/  
  MRF_F0::randomness_initialization();

  //  ofstream fic; // possible file for output instead of cout
  //  fic.open("details"); // opening file

  // This minimizes an energy generated randomly. 
  // Check the code for details and for minimizing your own energy.
  MRF_F0::minimize_random_familial_energy(); // with display options by default (outputs key steps and summaries)
  //MRF_F0::minimize_random_familial_energy(false, false, false, false, false, false, false, false, false, cout); // or : (..., fic); // for no output
  //MRF_F0::minimize_random_familial_energy(true, true, true, true, true, true, true, true, true,  cout); // you don't want that many detail !
  
  //    fic.close(); // closing file after output

  cout << endl << endl << "This is the end..." << endl << endl;



  /********* Minimizing an energy based on an image **********/

#ifdef I_want_to_use_the_graphical_library
  cout << endl << endl << MRF_F0::BoldRed << "  ***************  " << MRF_F0::BoldGreen << "Second Test" << MRF_F0::BoldRed << "  ***************" << MRF_F0::TextNormal << endl << endl;

  MRF_F0::minimize_on_an_image_grid(0);

  //  endGraphics(); // should be added at the end of the code, if and only if a window has been opened before
  cout << endl << endl << "This is the end (again)..." << endl << endl;
#endif


  return 0;
}


